%%
% LPV Example
%
% M-file to analyze an uncetrain linear parameter varying system using IQCs 
% and dissipation inequalities. The analysis is performed for various bounds
% on the parameter variation rate and compared to the wcgain command
% The LPV system is taken from "Time-Delay System", Weehon Tan, Berkeley, 
% Automatica vol33, 1997, pp411-419


%% 
% Setting up the plant model
clear all
yvec = (2:1:7); %scheduling parameter vector
K = sqrt(4.8*yvec-8.6); %parameter varying gain of the 1st order LPV system
T = sqrt(133.6-16.8*yvec); %time constant

% Adding a basic gain scheduled PI controller
zeta = .7; %CL damping requirement at each grid point
omega = .25; %CL frequency requirement at each grid point

Ki = omega.^2.*T./K; %Integrator gain
Kp = (2*zeta*omega.*T-1)./K; %proportinal gain

%build LPV plant, controller and closed loop
Nmod = length(yvec);
for ii = 1:Nmod
    P(:,:,ii)= ss(-1/T(ii),1/(T(ii)),K(ii),0);
    
    C(:,:,ii) = ss(0,Ki(ii),1,Kp(ii));
    
    clp(:,:,ii) = feedback(P(:,:,ii)*C(:,:,ii),1);
    
end

%time delay at the plant intput
Td = .5;
tdelay = tf([Td^2/12,-Td/2,1],[Td^2/12,Td/2,1]);


%% unbounded rates
clear giqclmi

% basis function for the parameter dependent lyapunov function
Fbasis = ones(1,1,Nmod); % constant term
Fgrad = zeros(1,0); %gradient of the basis function with respect to the parameters
% bounds on the parameter variation rates
RateBounds = []; %empty means unbounded

% uncertainty is norm bounded by kappa
kappavec = [0,0.01,0.05,0.10];%,.15,.2,.25,.3,.35,.4,.45,.5];

for ii = 1:length(kappavec)
    kappa = kappavec(ii);
    
    %Define IQCs for norm bounded LTI uncertainty
    M = blkdiag(kappa^2,-1);
    s=tf('s');
    psi1 = eye(2);
    psi2 =blkdiag(1/(s+1),1/(s+1));
    
    IQClist = {{psi1,M},{psi2,M}};
    
    
    %build plant interconnection
    plant = P;
    controller = C;
    systemnames = 'plant controller tdelay';
    inputvar = '[ v{1}; d{1} ]';
    outputvar = '[ controller; -plant(1) ]';
    input_to_plant = '[tdelay+d(1)]';
    input_to_tdelay = '[v + controller]';
    input_to_controller = '[-plant(1) ]';
    sysoutname = 'G';
    cleanupsysic = 'yes';
    sysic
    
    [giqclmi(1,ii),info] = simplenormboundedgain(G,IQClist,Fbasis,Fgrad,RateBounds);
    
end

%plot results
figure(1)
hold on
plot(kappavec,giqclmi(1,:),'+--','color',mycolor('maroon'),'MarkerSize',12)
xlim([0 0.8])
xlabel('norm bound b [-]')
ylabel('worst case gain \gamma [-]')
if exist('garyfyFigure','file')
    garyfyFigure;
end
box on


%% influence of basis functions on the performance results
clear giqclmi

% bounds on the parameter variation rates
RateBounds = [-2,2];

% uncertainty is norm bounded by kappa
kappavec = [0,0.01,0.05,0.10,.15,.2,.25,.3,.35,.4,.45,.5];

for ii = 1:length(kappavec)
    kappa = kappavec(ii);

    %Define IQCs for norm bounded LTI uncertainty
    M = blkdiag(kappa^2,-1);
    s=tf('s');
    psi1 = eye(2);
    psi2 =blkdiag(1/(s+1),1/(s+1));
    
    IQClist = {{psi1,M},{psi2,M}};

    %build plant interconnection
    plant = P;
    controller = C;
    systemnames = 'plant controller tdelay';
    inputvar = '[ v{1}; d{1} ]';
    outputvar = '[ controller; -plant(1) ]';
    input_to_plant = '[tdelay+d(1)]';
    input_to_tdelay = '[v + controller]';
    input_to_controller = '[-plant(1) ]';
    sysoutname = 'G';
    cleanupsysic = 'yes';
    sysic
    
    % basis function for the parameter dependent lyapunov function
    Fbasis = ones(2,1,6); %First term is constant
    Fbasis(2,1,:) = reshape(yvec',1,1,6); %second term linear in rho
    %gradient of the basis function with respect to the parameters
    Fgrad = zeros(2,1,6);
    Fgrad(2,1,:) = ones(1,1,6);
    [gam,info] = simplenormboundedgain(G,IQClist,Fbasis,Fgrad,RateBounds);

    if ~isempty(gam)
        giqclmi(1,ii) = gam;
    else
        giqclmi(1,ii) = 0;
    end
    
    % basis function for the parameter dependent lyapunov function
    Fbasis = ones(3,1,6); %First term is constant
    Fbasis(2,1,:) = reshape(yvec',1,1,6); %second term linear in rho
    Fbasis(3,1,:) = reshape(yvec'.^2,1,1,6); %third term quadratic in rho
    %gradient of the basis function with respect to the parameters
    Fgrad = zeros(2,1,6);
    Fgrad(2,1,:) = ones(1,1,6);
    Fgrad(3,1,:) = reshape(2*yvec',1,1,6);
   
    [gam,info] = simplenormboundedgain(G,IQClist,Fbasis,Fgrad,RateBounds);
    
    if ~isempty(gam)
        giqclmi(2,ii) = gam;
    else
        giqclmi(2,ii) = 0;
    end

    
end

%plot results
figure(2)
hold on
plot(kappavec(1:7),giqclmi(1,1:7),'+--','color',mycolor('maroon'),'MarkerSize',12)
plot(kappavec(1:9),giqclmi(2,1:9),'x-','color',mycolor('coolblue'),'MarkerSize',12)
xlim([0 0.8])
xlabel('norm bound b [-]')
ylabel('worst case gain \gamma [-]')
if exist('garyfyFigure','file')
    garyfyFigure;
end
box on


%% comparison with worst case

clear giqclmi

% uncertainty is norm bounded by kappa
kappavec = [0.01,0.05,0.10,.15,.2,.25,.3,.35,.4,.45,.5];

%at each grid point compute worst case LTI gain using wcgain.m
for jj = 1:length(kappavec)
    kappa=kappavec(jj);
    clear delta
    delta = ultidyn('delta',[1,1],'Bound',kappa);
    for ii = 1:length(yvec)
        
        plant =P(:,:,ii);
        controller = C(:,:,ii);
        
        systemnames = 'plant controller delta tdelay';
        inputvar = '[ d{1} ]';
        outputvar = '[ -plant(1) ]';
        input_to_plant =  '[tdelay+d(1)]';
        input_to_tdelay ='[delta+controller]';
        input_to_controller = '[-plant(1) ]';
        input_to_delta = '[controller]';
        sysoutname = 'sensii';
        cleanupsysic = 'yes';
        sysic
        
        [wcg{ii},wcu{ii},info] = wcgain(sensii);
        
        wcgjj(ii) = wcg{ii}.UpperBound;
        
    end
    
    %worst case gain for a given kappa over all grid points
    wcglti(jj+1) = max(wcgjj);
end

% bounds on the parameter variation rates
RateBounds = [-.1,.1];
% uncertainty is norm bounded by kappa
kappavec = [0,0.01,0.05,0.10,.15,.2,.25,.3,.35,.4,.45,.5];

for ii = 1:length(kappavec)
    kappa = kappavec(ii);

    %Define IQCs for norm bounded LTI uncertainty
    M = blkdiag(kappa^2,-1);
    s=tf('s');
    psi1 = eye(2);
    psi2 =blkdiag(1/(s+1),1/(s+1));
    
    IQClist = {{psi1,M},{psi2,M}};
    
    %build plant interconnection
    plant = P;
    controller = C;
    systemnames = 'plant controller tdelay';
    inputvar = '[ v{1}; d{1} ]';
    outputvar = '[ controller; -plant(1) ]';
    input_to_plant = '[tdelay+d(1)]';
    input_to_tdelay = '[v + controller]';
    input_to_controller = '[-plant(1) ]';
    sysoutname = 'G';
    cleanupsysic = 'yes';
    sysic
    
    % basis function for the parameter dependent lyapunov function
    Fbasis = ones(2,1,6); %First term is constant
    Fbasis(2,1,:) = reshape(yvec',1,1,6); %second term linear in rho
    %gradient of the basis function with respect to the parameters
    Fgrad = zeros(2,1,6);
    Fgrad(2,1,:) = ones(1,1,6);
    [gam,info] = simplenormboundedgain(G,IQClist,Fbasis,Fgrad,RateBounds);

    if ~isempty(gam)
        giqclmi(1,ii) = gam;
    else
        giqclmi(1,ii) = 0;
    end
    
    
    % basis function for the parameter dependent lyapunov function
    Fbasis = ones(3,1,6); %First term is constant
    Fbasis(2,1,:) = reshape(yvec',1,1,6); %second term linear in rho
    Fbasis(3,1,:) = reshape(yvec'.^2,1,1,6); %third term quadratic in rho
    %gradient of the basis function with respect to the parameters
    Fgrad = zeros(2,1,6);
    Fgrad(2,1,:) = ones(1,1,6);
    Fgrad(3,1,:) = reshape(2*yvec',1,1,6);
    [gam,info] = simplenormboundedgain(G,IQClist,Fbasis,Fgrad,RateBounds);
    
    if ~isempty(gam)
        giqclmi(2,ii) = gam;
    else
        giqclmi(2,ii) = 0;
    end

    
end

%plot results
figure(3)
hold on
plot(kappavec,giqclmi(1,:),'+--','color',mycolor('maroon'),'MarkerSize',12)
plot(kappavec,giqclmi(2,:),'x-','color',mycolor('coolblue'),'MarkerSize',12)
plot(kappavec(2:end),wcglti(2:end),'o:','color',mycolor('darkgray'),'MarkerSize',8)
xlim([0 0.5])
xlabel('norm bound on uncertainty b [-]')
ylabel('worst case gain \gamma [-]')
legend('P affine','P quadratic','wcgain');
if exist('garyfyFigure','file')
    garyfyFigure;
end
box on
legend('Location','NorthWest')





